#!/bin/bash
#
# Mass Storage Cloner - blkwrite.sh
# (C)2015 Martin Suefke
# License: GPLv3 or newer
#
# writes source file $1 to block device $2
#
# DO NOT Customize this file. Try to customize write.sh first
#

set -e      # break if error

function onerror() {
    echo
    echo "Trapped error while copying >$SRC< to >$TGT< Result code is \"$res\"".
    echo -n "Press Enter to Exit this script ..."
    read -n1
    trap EXIT
    exit 1
}

trap onerror ERR EXIT

echo "Writing >$1< to >$2<"
if [ $# -lt 2 ]
then
  echo "Need 2 parameters: source target"
  exit 1
fi

SRC="$1"
TGT="$2"

if [ ! -r "$SRC" ]
then
    echo "Source file >$SRC< is not readable. Exit."
    exit 2
fi

if [ ! -b "$TGT" ]
then
    echo "need a blockdevice as parameter 2. Exit."
    exit 3
fi

if [ "x${TGT%%[0-9]*}" == "x$TGT" ]
then
  # have a "master" block device
  echo "removing partitions from $TGT with partx"
  partx -d "$TGT" || true
fi

echo "Start copy operation"
# live version
pv -trabep -B8m $SRC | dd bs=8M iflag=fullblock of="$TGT" oflag=sync
# debug version
#pv -trabep -B8m $SRC | dd bs=8M iflag=fullblock of=/dev/null oflag=sync
res="$?"
echo "Result code $res"

if [ "x${TGT%%[0-9]*}" == "x$TGT" ]
then
  # have a "master" block device
  echo "adding partitions for $TGT with partx"
  partx -a "$TGT" || true
fi

if [ x$res == x0 ]
then
    trap EXIT
else
    onerror
fi
#end;
